searchData={"items":[{"type":"module","title":"Fey.Enum","doc":"`Fey.Enum` module provides alternatives to some functions from `Enum` module from the standard\nlibrary, returning option informing about the result of the lookup. This allows, among others,\nto distinguish between situation when the found value is `nil` and when the value is not found.","ref":"Fey.Enum.html"},{"type":"function","title":"Fey.Enum.at/2","doc":"Alternative to `Enum.at/2` which returns an option: `{:some, value}` if there is a value\nat given index or `:none` if there isn't such element.","ref":"Fey.Enum.html#at/2"},{"type":"function","title":"Examples - Fey.Enum.at/2","doc":"iex> Fey.Enum.at([1, nil, 2], 1)\n    {:some, nil}\n\n    iex> Fey.Enum.at([1, 2, 3], 5)\n    :none\n\n    iex> Fey.Enum.at([1, nil, :none], 2)\n    {:some, :none}\n\n    iex> Fey.Enum.at(%{a: 1, b: 2}, 1)\n    {:some, {:b, 2}}","ref":"Fey.Enum.html#at/2-examples"},{"type":"function","title":"Fey.Enum.find/2","doc":"Alternative to `Enum.find` which returns an option: `{:some, value}` if a value matching the\ncondition is found or `:none` if no such value was found.","ref":"Fey.Enum.html#find/2"},{"type":"function","title":"Examples - Fey.Enum.find/2","doc":"iex> Fey.Enum.find([1, 2, 3], &rem(&1, 2) == 0)\n    {:some, 2}\n\n    iex> Fey.Enum.find([1, 3, 5], &rem(&1, 2) == 0)\n    :none\n\n    iex> Fey.Enum.find(%{a: 1, b: 2}, fn {_, v} -> rem(v, 2) == 0 end)\n    {:some, {:b, 2}}","ref":"Fey.Enum.html#find/2-examples"},{"type":"module","title":"Fey.Keyword","doc":"","ref":"Fey.Keyword.html"},{"type":"function","title":"Fey.Keyword.get/2","doc":"Alternative to `Keyword.get/2`, but returns an option instead of value-or-nil. It allows\nto distinguish whether the key was not found at all or it was there, but the value was `nil`.\n\nIf you prefer result over option, see: `Fey.KeywordR.get/2`.\n\nNOTE: As the exception when passed argument is not a keyword leaks implementation details,\nit's likely to change in the future. This might not be considered a breaking change.","ref":"Fey.Keyword.html#get/2"},{"type":"function","title":"Examples - Fey.Keyword.get/2","doc":"iex> Fey.Keyword.get([a: 1, b: nil], :a)\n    {:some, 1}\n\n    iex> Fey.Keyword.get([a: 1, b: nil], :b)\n    {:some, nil}\n\n    iex> Fey.Keyword.get([a: 1, b: nil], :c)\n    :none\n\n    iex> Fey.Keyword.get(%{a: 1}, :a)\n    ** (FunctionClauseError) no function clause matching in Keyword.has_key?/2","ref":"Fey.Keyword.html#get/2-examples"},{"type":"module","title":"Fey.KeywordR","doc":"","ref":"Fey.KeywordR.html"},{"type":"function","title":"Fey.KeywordR.get/2","doc":"Alternative to `Keyword.get/2` (and `Fey.Keyword.get/2`), but returns an option instead of value-or-nil. It allows\nto distinguish whether the key was not found at all or it was there, but the value was `nil`.\n\nNOTE: As the exception when passed argument is not a keyword leaks implementation details,\nit's likely to change in the future. This might not be considered a breaking change.","ref":"Fey.KeywordR.html#get/2"},{"type":"function","title":"Examples - Fey.KeywordR.get/2","doc":"iex> Fey.Keyword.get([a: 1, b: nil], :a)\n    {:some, 1}\n\n    iex> Fey.Keyword.get([a: 1, b: nil], :b)\n    {:some, nil}\n\n    iex> Fey.Keyword.get([a: 1, b: nil], :c)\n    :none\n\n    iex> Fey.Keyword.get(%{a: 1}, :a)\n    ** (FunctionClauseError) no function clause matching in Keyword.has_key?/2","ref":"Fey.KeywordR.html#get/2-examples"},{"type":"module","title":"Fey.Map","doc":"","ref":"Fey.Map.html"},{"type":"function","title":"Fey.Map.get/2","doc":"Alternative to `map[key]`, but returns an option instead of value-or-nil. It allows\nto distinguish whether the key was not found at all or it was there, but the value was `nil`.\n\nIf you prefer results over options, see `Fey.MapR.get/2`.","ref":"Fey.Map.html#get/2"},{"type":"function","title":"Examples - Fey.Map.get/2","doc":"iex> Fey.Map.get(%{a: 1, b: nil}, :a)\n    {:some, 1}\n\n    iex> Fey.Map.get(%{a: 1, b: nil}, :b)\n    {:some, nil}\n\n    iex> Fey.Map.get(%{a: 1, b: nil}, :c)\n    :none\n\n    iex> Fey.Map.get([1,2,3], :x)\n    ** (BadMapError) expected a map, got: [1, 2, 3]","ref":"Fey.Map.html#get/2-examples"},{"type":"module","title":"Fey.MapR","doc":"","ref":"Fey.MapR.html"},{"type":"function","title":"Fey.MapR.get/2","doc":"Alternative to `map[key]` (and `Fey.Map.get`), but returns a result tuple instead of value-or-nil.\nIt allows to distinguish whether the key was not found at all or it was there, but the value was `nil`.\nIf the key wasn't in the map, returns `{:error, :not_found}`.\n\nIf you prefer option, see `Fey.Map.get/2`.","ref":"Fey.MapR.html#get/2"},{"type":"function","title":"Examples - Fey.MapR.get/2","doc":"iex> Fey.MapR.get(%{a: 1, b: nil}, :a)\n    {:ok, 1}\n\n    iex> Fey.MapR.get(%{a: 1, b: nil}, :b)\n    {:ok, nil}\n\n    iex> Fey.MapR.get(%{a: 1, b: nil}, :c)\n    {:error, :not_found}\n\n    iex> Fey.MapR.get([1,2,3], :x)\n    ** (BadMapError) expected a map, got: [1, 2, 3]","ref":"Fey.MapR.html#get/2-examples"},{"type":"module","title":"Fey.Option","doc":"`Option` module serves a similar, but less popular in Elixir, pattern. Instead of `{:ok, something}`\nor `{:error, message}` tuples, `Option` works with `{:some, value}` or `:none`.\n\nUnlike `Result`, which is best used to describe the results of some operation, that might have\nfinished successfully or error out, `Option` is more geared towards working with optional\ndata.\n\nIt might be a good replacement of `nil`. Let's say that we have a function returning nth\nelement of some list:\n\n```elixir\ndef nth(list, idx), do: Enum.at(list, idx)\n```\n\nThis works well, but you have no way to distinguish whether the index you asked for was\nout of bounds or simply the value at this point was `nil`. This could be solved by using\n`Option`:\n\n```elixir\ndef nth([head | _], 0), do: {:some, head}\ndef nth([], _), do: :none\ndef nth[head | tail], idx), do: nth(tail, idx - 1)\n\niex> nth([1, nil, 3], 1)\n{:some, nil}\n\niex> nth([1, nil, 3], 4)\n:none\n```\n\nThis is obviously a very simplified example, but it shows why the difference might matter.\nIn \"real\" applications it's often the difference between \"not yet fetched/calculated\" and\n\"fetched/calculated, but has no value\".","ref":"Fey.Option.html"},{"type":"function","title":"Fey.Option.bind/2","doc":"If the option is some, returns a value of fun applied on option's value. It's analogous to `map/2`, but\ndoes not wrap in `{:some, val}` automatically, which makes it useful if `fun` returns an option.\n\nIf the option is none, returns none.\n\nRaises `Fey.Option.BadArgument` if the value passed as argument is not a valid option.","ref":"Fey.Option.html#bind/2"},{"type":"function","title":"Examples - Fey.Option.bind/2","doc":"iex> Fey.Option.bind({:some, 42}, fn v -> {:some, v/2} end)\n    {:some, 21.0}\n\n    iex> Fey.Option.bind({:some, 42}, fn v -> v/2 end)\n    21.0\n\n    iex> Fey.Option.bind(:none, fn v -> {:some, v/2} end)\n    :none\n\n    iex> Fey.Option.bind(15, fn v -> {:some, v/2} end)\n    ** (Fey.Option.BadArgument) 15 is not a valid option","ref":"Fey.Option.html#bind/2-examples"},{"type":"function","title":"Fey.Option.get!/1","doc":"If the option is some, returns its value.\n\nRaises `Fey.Option.NotSome` if the result is not some.\nRaises `Fey.Option.BadArgument` if the value is not a valid option tuple.","ref":"Fey.Option.html#get!/1"},{"type":"function","title":"Examples - Fey.Option.get!/1","doc":"iex> Fey.Option.get!({:some, 42})\n    42\n\n    iex> Fey.Option.get!(:none)\n    ** (Fey.Option.NotSome) :none is not some\n\n    iex> Fey.Option.get!(nil)\n    ** (Fey.Option.BadArgument) nil is not a valid option","ref":"Fey.Option.html#get!/1-examples"},{"type":"function","title":"Fey.Option.get_with_default/2","doc":"If the option is some, return its value, but if the option is non, return a default value\ninstead of raising exception (like `get!/1` does).\n\nRaises `Fet.Option.BadArgument` if the values passed as argument is not a valid option.","ref":"Fey.Option.html#get_with_default/2"},{"type":"function","title":"Examples - Fey.Option.get_with_default/2","doc":"iex> Fey.Option.get_with_default({:some, 42}, 10)\n    42\n\n    iex> Fey.Option.get_with_default(:none, 10)\n    10\n\n    iex> Fey.Option.get_with_default(\"string\", 10)\n    ** (Fey.Option.BadArgument) \"string\" is not a valid option","ref":"Fey.Option.html#get_with_default/2-examples"},{"type":"function","title":"Fey.Option.map/2","doc":"If the option is some, returns `{:some, val}`, where `val` is a return value of fun applied on option's value.\n\nIf the option is none, returns `:none`.\n\nRaises `Fey.Option.BadArgument` if the value passed as argument is not a valid option.","ref":"Fey.Option.html#map/2"},{"type":"function","title":"Examples - Fey.Option.map/2","doc":"iex> Fey.Option.map({:some, 42}, fn v -> v * 2 end)\n    {:some, 84}\n\n    iex> Fey.Option.map(:none, fn v -> v * 2 end)\n    :none\n\n    iex> Fey.Option.map(55, fn v -> v * 2 end)\n    ** (Fey.Option.BadArgument) 55 is not a valid option","ref":"Fey.Option.html#map/2-examples"},{"type":"function","title":"Fey.Option.none?/1","doc":"Returns whether or not the passed values is none.\nRaises `Fey.Option.BadArgument` if the values is not a valid option","ref":"Fey.Option.html#none?/1"},{"type":"function","title":"Examples - Fey.Option.none?/1","doc":"iex> Fey.Option.none?(:none)\n    true\n\n    iex> Fey.Option.none?({:some, 42})\n    false\n\n    iex> Fey.Option.none?(nil)\n    ** (Fey.Option.BadArgument) nil is not a valid option","ref":"Fey.Option.html#none?/1-examples"},{"type":"function","title":"Fey.Option.some?/1","doc":"Returns whether or not the passed value is some.\nRaises `Fey.Option.BadArgument` if the value is not a valid option.","ref":"Fey.Option.html#some?/1"},{"type":"function","title":"Examples - Fey.Option.some?/1","doc":"iex> Fey.Option.some?({:some, 42})\n    true\n\n    iex> Fey.Option.some?(:none)\n    false\n\n    iex> Fey.Option.some?(nil)\n    ** (Fey.Option.BadArgument) nil is not a valid option","ref":"Fey.Option.html#some?/1-examples"},{"type":"function","title":"Fey.Option.wrap/1","doc":"Wraps a value in option tuple (i.e. `{:some, value}`)","ref":"Fey.Option.html#wrap/1"},{"type":"function","title":"Examples - Fey.Option.wrap/1","doc":"iex> Fey.Option.wrap(true)\n    {:some, true}\n\n    iex> Fey.Option.wrap(nil)\n    {:some, nil}\n\n    iex> Fey.Option.wrap({:some, 15})\n    {:some, {:some, 15}}","ref":"Fey.Option.html#wrap/1-examples"},{"type":"function","title":"Fey.Option.wrap_not_nil/1","doc":"Similar to `wrap/1`, but only returns a some-tuple if the passed value is not `nil`.\nOtherwise `:none` is returned.\n\nNote that generally it's preferred over `Fey.Result.wrap_not_nil/2`.","ref":"Fey.Option.html#wrap_not_nil/1"},{"type":"function","title":"Examples - Fey.Option.wrap_not_nil/1","doc":"iex> Fey.Option.wrap_not_nil(42)\n    {:some, 42}\n\n    iex> Fey.Option.wrap_not_nil(nil)\n    :none","ref":"Fey.Option.html#wrap_not_nil/1-examples"},{"type":"type","title":"Fey.Option.none_/0","doc":"","ref":"Fey.Option.html#t:none_/0"},{"type":"type","title":"Fey.Option.some/0","doc":"","ref":"Fey.Option.html#t:some/0"},{"type":"type","title":"Fey.Option.t/0","doc":"","ref":"Fey.Option.html#t:t/0"},{"type":"module","title":"Fey.Result","doc":"The module provides functions to work with a well-established pattern of result tuples\n(sometimes called \"tagged tuples\") in Elixir. By them we mean `{:ok, value}` or `{:error, error}`\ntuples.\n\n`Fey.Result` adds a bunch of convenience to work with these, especially in pipelines. They can replace\n`with` chains, which are frowned upon by some people in the Elixir community. Let's see an example:\n\n```elixir\nRepo.fetch(OrderLineItem, 123)\n|> Fey.Result.map(& &1.sku)\n|> Fey.Result.bind(&Repo.get_by(Product, sku: &1) |> Fey.Result.wrap_not_nil(:sku_not_found))\n|> Fey.Result.map(& &1.name)\n```\n\nThis either returns a name of the product for line item (matches by SKU) or any of the intermediate\nerrors that might happpen: `{:ok, :not_found}` (from `Repo.fetch`) or `{:error, :sku_not_found}`.","ref":"Fey.Result.html"},{"type":"function","title":"Fey.Result.bind/2","doc":"If the result is a success, returns a value of fun applied on result's value. It's analogous to `map/2`, but does not wrap\nin `{:ok, val}` automatically, which makes it useful if fun returns a result.\n\nIf the result is an error, returns error itself.\n\nRaises `Fey.Result.BadArgument` if the value passed as argument is not a valid result tuple.","ref":"Fey.Result.html#bind/2"},{"type":"function","title":"Examples - Fey.Result.bind/2","doc":"iex> Fey.Result.bind({:ok, 42}, fn v -> {:ok, v / 2} end)\n    {:ok, 21.0}\n\n    iex> Fey.Result.bind({:ok, 42}, fn v -> v / 2 end)\n    21.0\n\n    iex> Fey.Result.bind({:error, :not_found}, fn v -> {:ok, v / 2} end)\n    {:error, :not_found}\n\n    iex> Fey.Result.bind(:some_atom, fn v -> {:ok, v / 2} end)\n    ** (Fey.Result.BadArgument) :some_atom is not a valid result tuple","ref":"Fey.Result.html#bind/2-examples"},{"type":"function","title":"Fey.Result.bind_error/2","doc":"Inverse of `bind/2` - if the result is an error, executes fun and returns its result.\n\nIf the result is success, just returns it.\n\nRaises `Fey.Result.BadArgument` if the value passed as argument is not a valid result tuple.\n\nThis function makes most sense when used in a pipeline where you want to\ntake the first successful result. For example:\n\n```elixir\nparse_as_iso_datetime(input)\n|> Fey.Result.bind_error(fn -> parse_as_iso_date(input) end)\n|> Fey.Result.bind_error(fn -> parse_as_naive_datetime(input) end)\n|> Fey.Result.bind_error(fn -> parse_as_naive_date(input) end)\n\n# {:ok, parsed_date} | {:error, :bad_format}\n```","ref":"Fey.Result.html#bind_error/2"},{"type":"function","title":"Examples - Fey.Result.bind_error/2","doc":"iex> Fey.Result.bind_error({:error, :not_found}, fn -> {:ok, 42} end)\n    {:ok, 42}\n\n    iex> Fey.Result.bind_error({:ok, nil}, fn -> {:ok, 42} end)\n    {:ok, nil}\n\n    iex> Fey.Result.bind_error(nil, fn -> {:ok, 42} end)\n    ** (Fey.Result.BadArgument) nil is not a valid result tuple","ref":"Fey.Result.html#bind_error/2-examples"},{"type":"function","title":"Fey.Result.error!/1","doc":"If the result is an error, return its details.\n\nRaises NotError if the result is not errorneous.\n\nRaises `Fey.Result.BadArgument` is the value passed as argument is not a valid result tuple.","ref":"Fey.Result.html#error!/1"},{"type":"function","title":"Examples - Fey.Result.error!/1","doc":"iex> Fey.Result.error!({:error, :not_found})\n    :not_found\n\n    iex> Fey.Result.error!({:ok, 42})\n    ** (Fey.Result.NotError) {:ok, 42} is not an error\n\n    iex> Fey.Result.error!(:some_atom)\n    ** (Fey.Result.BadArgument) :some_atom is not a valid result tuple","ref":"Fey.Result.html#error!/1-examples"},{"type":"function","title":"Fey.Result.error?/1","doc":"Returns whether or not the passed value is an error.\n\nRaises Fey.Result.BadArgument if the value is not a valid result tuple.","ref":"Fey.Result.html#error?/1"},{"type":"function","title":"Examples - Fey.Result.error?/1","doc":"iex> Fey.Result.error?({:ok, true})\n    false\n\n    iex> Fey.Result.error?({:error, :not_found})\n    true\n\n    iex> Fey.Result.error?(42)\n    ** (Fey.Result.BadArgument) 42 is not a valid result tuple","ref":"Fey.Result.html#error?/1-examples"},{"type":"function","title":"Fey.Result.get!/1","doc":"If the result is a success, returns its value.\n\nRaises `Fey.Result.NotSuccess` if the result is not successful.\nRaises `Fey.Result.BadArgument` if the value passed as argument is not a valid result tuple.","ref":"Fey.Result.html#get!/1"},{"type":"function","title":"Examples - Fey.Result.get!/1","doc":"iex> Fey.Result.get!({:ok, 42})\n    42\n\n    iex> Fey.Result.get!({:error, :not_found})\n    ** (Fey.Result.NotSuccess) {:error, :not_found} is not a success\n\n    iex> Fey.Result.get!(:some_atom)\n    ** (Fey.Result.BadArgument) :some_atom is not a valid result tuple","ref":"Fey.Result.html#get!/1-examples"},{"type":"function","title":"Fey.Result.get_with_default/2","doc":"If the result is a success, return its value but if the result is an error, return default value\ninstead of raising exception (like `get!/1` does).\n\nRaises `Fey.Result.BadArgument` if the value passed as argument is not a valid result tuple.","ref":"Fey.Result.html#get_with_default/2"},{"type":"function","title":"Examples - Fey.Result.get_with_default/2","doc":"iex> Fey.Result.get_with_default({:ok, 42}, 1567)\n    42\n\n    iex> Fey.Result.get_with_default({:error, :not_found}, 1567)\n    1567\n\n    iex> Fey.Result.get_with_default(\"string\", 1567)\n    ** (Fey.Result.BadArgument) \"string\" is not a valid result tuple","ref":"Fey.Result.html#get_with_default/2-examples"},{"type":"function","title":"Fey.Result.map/2","doc":"If the result is a success, returns `{:ok, val}`, where `val` is a return value of fun applied on result's value.\n\nIf the result is an error, returns error itself.\n\nRaises `Fey.Result.BadArgument` if the value passed as argument is not a valid result tuple.","ref":"Fey.Result.html#map/2"},{"type":"function","title":"Examples - Fey.Result.map/2","doc":"iex> Fey.Result.map({:ok, 42}, fn v -> v / 2 end)\n    {:ok, 21.0}\n\n    iex> Fey.Result.map({:error, :not_found}, fn v -> v / 2 end)\n    {:error, :not_found}\n\n    iex> Fey.Result.map(:some_atom, fn v -> v / 2 end)\n    ** (Fey.Result.BadArgument) :some_atom is not a valid result tuple","ref":"Fey.Result.html#map/2-examples"},{"type":"function","title":"Fey.Result.ok?/1","doc":"Returns whether or not the passed value is a success.\nRaises Fey.Result.BadArgument if the value is not a valid result tuple.","ref":"Fey.Result.html#ok?/1"},{"type":"function","title":"Examples - Fey.Result.ok?/1","doc":"iex> Fey.Result.ok?({:ok, true})\n    true\n\n    iex> Fey.Result.ok?({:error, :not_found})\n    false\n\n    iex> Fey.Result.ok?(42)\n    ** (Fey.Result.BadArgument) 42 is not a valid result tuple","ref":"Fey.Result.html#ok?/1-examples"},{"type":"function","title":"Fey.Result.wrap/1","doc":"Wraps a value in a success result tuple (i.e. `{:ok, value}`).\n\nNote that it does not check whether a value passed as argument is already a valid\nresult tuple, or not.","ref":"Fey.Result.html#wrap/1"},{"type":"function","title":"Examples - Fey.Result.wrap/1","doc":"iex> Fey.Result.wrap(true)\n    {:ok, true}\n\n    iex> Fey.Result.wrap({:ok, \"fourty two\"})\n    {:ok, {:ok, \"fourty two\"}}","ref":"Fey.Result.html#wrap/1-examples"},{"type":"function","title":"Fey.Result.wrap_not_nil/2","doc":"Similar to `wrap/1`, but only returns a success result if the passed value is not `nil`.\nIf it is, `{:error, error}` is returned, where `error` is either passed as a second\nargument, or by default is `:not_found`.\n\nNote that generally `Fey.Option.wrap_not_nil/1` is preferred, as it's more idiomatic.\nHowever, if you're set on using `Result`, this is available.","ref":"Fey.Result.html#wrap_not_nil/2"},{"type":"function","title":"Examples - Fey.Result.wrap_not_nil/2","doc":"iex> Fey.Result.wrap_not_nil(42)\n    {:ok, 42}\n\n    iex> Fey.Result.wrap_not_nil(nil)\n    {:error, :not_found}\n\n    iex> Fey.Result.wrap_not_nil(nil, :number_missing)\n    {:error, :number_missing}","ref":"Fey.Result.html#wrap_not_nil/2-examples"},{"type":"type","title":"Fey.Result.error/0","doc":"","ref":"Fey.Result.html#t:error/0"},{"type":"type","title":"Fey.Result.success/0","doc":"","ref":"Fey.Result.html#t:success/0"},{"type":"type","title":"Fey.Result.t/0","doc":"","ref":"Fey.Result.html#t:t/0"},{"type":"exception","title":"Fey.Option.BadArgument","doc":"","ref":"Fey.Option.BadArgument.html"},{"type":"exception","title":"Fey.Option.NotNone","doc":"","ref":"Fey.Option.NotNone.html"},{"type":"exception","title":"Fey.Option.NotSome","doc":"","ref":"Fey.Option.NotSome.html"},{"type":"exception","title":"Fey.Result.BadArgument","doc":"","ref":"Fey.Result.BadArgument.html"},{"type":"exception","title":"Fey.Result.NotError","doc":"","ref":"Fey.Result.NotError.html"},{"type":"exception","title":"Fey.Result.NotSuccess","doc":"","ref":"Fey.Result.NotSuccess.html"},{"type":"extras","title":"Fey","doc":"# Fey\n\nFey is a set of functions to work with result tuples and their counterparts - option values. It is heavily inspired by OCaml naming conventions and built for people who like their pipe operator (perhaps too much).\n\n``` elixir\nRepo.get(DataFile, id)\n|> Fey.Result.wrap_not_nil()\n|> Fey.Result.map(fn df -> df.filename end)\n|> Fey.Result.bind(fn fname -> File.open(fname) end)\n|> Fey.Result.bind(fn contents -> Jason.decode(contents) end)\n|> Fey.Map.get(\"score\")\n|> Fey.Result.get_with_default(0.0)\n```\n\nThis code reads the records from the database, wraps it in the result tuple and then, only when the record was found, performs a bunch of operations on it.\n\nThis is a classic railway-oriented programming, which could just as well be modelled using the Elixir-native `with` clause. However some people don't like using `with` and in this case it might be not so nice, as not all the steps return proper result tuple. For example, its beginning would have to be:\n\n``` elixir\nwith record when not is_nil(record) <- Repo.get(DataFile, id),\n```\n\nFey offers a different approach: use a regular function that returns data-or-nil and then call `Fey.Result.wrap_not_nil/1` on it, which results in either `{:ok, record}` or `{:error, :not_found}`. Then it's taken through a pipeline of functions operating on the results ~~monad~~[^1] tuple.\n\n### \"Reversed with\"\n\nAnother case where `Fey` could help is with so-called \"reversed with\" (or \"sad path with\") where you execute bunch of operations until the first success. Let's say we have an IP address and we want to check 3 different APIs to check if it should not be banned. The code could look like this:\n\n``` elixir\ndef banned?(ip) do\n  with {:error, :not_found} <- check_api1(ip),\n      {:error, :not_found} <- check_api2(ip),\n      {:error, :not_found} <- check_api3(ip) do\n    false\n  else\n    {:ok, _reason} -> true\n  end\nend\n```\n\nWith `Fey` it would look like this:\n\n``` elixir\ndef banned?(ip) do\n  check_api1(ip)\n  |> Fey.Result.bind_error(fn -> check_api2(ip) end)\n  |> Fey.Result.bind_error(fn -> check_api3(ip) end)\n  |> Fey.Result.error?()\nend\n```","ref":"readme.html"},{"type":"extras","title":"Option vs Result - Fey","doc":"Fey encourages using option ~~monad~~ instead of result where it makes sense. Result is best when we are talking about a result of an operation, however option works better when we lookup something.\n\nAnd option, similar to result, is either a `{:some, value}` tuple or `:none` atom. We can see it in action:\n\n``` elixir\n[1, 2, 3]\n|> Fey.Enum.find(& rem(&1, 2) == 0)\n#=> {:some, 2}\n\n[1, 2, 3]\n|> Fey.Enum.find(& &1 > 5\n#=> :none\n```","ref":"readme.html#option-vs-result"},{"type":"extras","title":"Installation - Fey","doc":"The package can be installed by adding `fey` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:fey, \"~> 0.0.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Footnotes - Fey","doc":"[^1]: Ooops, I said the M-word...","ref":"readme.html#footnotes"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}